@@ -21,12 +21,15 @@ int main (int argc, char **argv, char **env) {
     char *result, *error;
     char cmd[120];
     char hostname[GM_BUFFERSIZE];
+    char cwd[1024];
     struct stat st;
 
-    plan(60);
+    plan(74);
 
-    /* set hostname */
+    /* set hostname and cwd */
     gethostname(hostname, GM_BUFFERSIZE-1);
+    if(!getcwd(cwd, sizeof(cwd)) != NULL)
+       perror("getcwd() error");
 
     /* create options structure and set debug level */
     mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
@@ -137,6 +140,8 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(rc, "==", 0, "pclose for cmd '%s' returned rc %d", cmd, rc);
     rrc = real_exit_code(rc);
     cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "^USERS OK", "returned result string");
+    like(error, "^$", "returned no errors");
     free(result);
     free(error);
 
@@ -168,9 +173,13 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     fork_on_exec = 1;
     lives_ok({execute_safe_command(exec_job, fork_on_exec, hostname);}, "executing command using fork on exec");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* non existing command 2 */
     free(exec_job->command_line);
@@ -178,6 +187,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
 
 
@@ -191,6 +202,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* non existing command 2 */
     fork_on_exec = 1;
@@ -199,6 +212,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
 
 
@@ -211,6 +226,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 5 is out of bounds. \\(worker:.*exiting with exit code 5", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
 
     /* unknown exit code 2 */
@@ -220,6 +237,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 5 is out of bounds. \\(worker:.*exiting with exit code 5", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* unknown exit code 3 */
     free(exec_job->command_line);
@@ -227,6 +246,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 128 is out of bounds. Plugin exited by signal signal 0. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* unknown exit code 4 */
     free(exec_job->command_line);
@@ -234,6 +255,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 137 is out of bounds. Plugin exited by signal SIGKILL. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* unknown exit code 5 */
     free(exec_job->command_line);
@@ -241,6 +264,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 255 is out of bounds. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
 
     /*****************************************
@@ -253,6 +278,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 130 is out of bounds. Plugin exited by signal SIGINT. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* signaled exit code SIGINT 2 */
     fork_on_exec = 0;
@@ -261,6 +288,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "CRITICAL: Return code of 130 is out of bounds. Plugin exited by signal SIGINT. \\(worker:", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
 
 
@@ -279,6 +308,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "\\(Service Check Timed Out On Worker: ", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /* timed out check 2 */
     fork_on_exec = 0;
@@ -287,6 +318,8 @@ int main (int argc, char **argv, char **env) {
     execute_safe_command(exec_job, fork_on_exec, hostname);
     cmp_ok(exec_job->return_code, "==", 2, "cmd '%s' returns rc 2", exec_job->command_line);
     like(exec_job->output, "\\(Service Check Timed Out On Worker: ", "returned result string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     signal(SIGTERM, SIG_DFL);
     signal(SIGINT, SIG_DFL);
@@ -305,6 +338,8 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(exec_job->return_code, "==", 0, "cmd '%s' returns rc 0", exec_job->command_line);
     like(exec_job->output, "out", "returned result string");
     like(exec_job->error,  "err", "returned error string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     free(exec_job->command_line);
     exec_job->command_line = strdup("./t/both;");
@@ -312,6 +347,8 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(exec_job->return_code, "==", 0, "cmd '%s' returns rc 0", exec_job->command_line);
     like(exec_job->output, "out", "returned result string");
     like(exec_job->error,  "err", "returned error string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     fork_on_exec = 1;
     free(exec_job->command_line);
@@ -320,6 +357,8 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(exec_job->return_code, "==", 0, "cmd '%s' returns rc 0", exec_job->command_line);
     like(exec_job->output, "out", "returned result string");
     like(exec_job->error,  "err", "returned error string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     free(exec_job->command_line);
     exec_job->command_line = strdup("./t/both;");
@@ -327,6 +366,8 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(exec_job->return_code, "==", 0, "cmd '%s' returns rc 0", exec_job->command_line);
     like(exec_job->output, "out", "returned result string");
     like(exec_job->error,  "err", "returned error string");
+    free(exec_job->output);
+    free(exec_job->error);
 
     /*****************************************
      * cmd env
@@ -337,6 +378,55 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(exec_job->return_code, "==", 0, "cmd '%s' returns rc 0", exec_job->command_line);
     like(exec_job->output, "test plugin OK", "returned result string");
 
+    /*****************************************
+     * restricted paths
+     */
+    char res[150];
+    snprintf(res, 150, "--restrict_path=/tmp/");
+    rc = parse_args_line(mod_gm_opt, res, 0);
+    cmp_ok(rc, "==", GM_OK, "parsed %s option", res);
+    cmp_ok(mod_gm_opt->restrict_path_num, "==", 1, "restricted path is set in opts: %d", mod_gm_opt->restrict_path_num);
+    strcpy(cmd, "./t/ok.pl");
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: restricted paths in affect, but command does not start with an absolute path: ./t/ok.p...", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (2)
+     */
+    strcpy(cmd, "./t/ok.pl; somethingnasty");
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: restricted paths in affect, but command does not start with an absolute path: ./t/ok.p...", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (3)
+     */
+    snprintf(res, 150, "--restrict_path=%s", cwd);
+    rc = parse_args_line(mod_gm_opt, res, 0);
+    cmp_ok(rc, "==", GM_OK, "parsed %s option", res);
+    cmp_ok(mod_gm_opt->restrict_path_num, "==", 2, "restricted path is set in opts: %d", mod_gm_opt->restrict_path_num);
+    snprintf(cmd, sizeof(cmd), "%s/t/ok.pl", cwd);
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "test plugin OK", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (4)
+     */
+    strcpy(cmd, "/forbidden/t/ok.pl");
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: command does not start with any of the restricted paths: /forbidd...", "returned result string");
+    free(result);
+    free(error);
+
     /*****************************************
      * clean up
      */
